#!/usr/bin/python

from .abstractmachine import AbstractCryptMachine
from secretpy import alphabets


class CryptMachine(AbstractCryptMachine):
    def __init__(self, cipher, key=None, alphabet=alphabets.ENGLISH):
        self.__alphabet = alphabet
        self.__key = key or ""
        self.__cipher = cipher

    def set_key(self, key):
        self.__key = key

    def set_alphabet(self, alphabet=alphabets.ENGLISH):
        self.__alphabet = alphabet

    def set_cipher(self, cipher):
        self.__cipher = cipher

    def encrypt(self, text):
        return self.__cipher.encrypt(text, self.__key, self.__alphabet)

    def decrypt(self, text):
        return self.__cipher.decrypt(text, self.__key, self.__alphabet)



#!/usr/bin/python
# -*- encoding: utf-8 -*-


class Vigenere:
    """
    The Vigenere Cipher
    """

    def __encDec(self, alphabet, key, text, isEncrypt):
        ans = ""
        for i in range(len(text)):
            char = text[i]
            keychar = key[i % len(key)]
            alphIndex = (alphabet.index(char) +
                         isEncrypt * alphabet.index(keychar)) % len(alphabet)
            ans += alphabet[alphIndex]
        return ans

    def encrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, 1)

    def decrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, -1)



#!/usr/bin/python

from .cryptmachine import CryptMachine
from .compositemachine import CompositeMachine
from .ciphers import *
from .alphabets import *
from .cmdecorators import *

__all__ = [
    "alphabets", "ciphers", "cmdecorators", "CompositeMachine", "CryptMachine",
]



#!/usr/bin/python
# -*- encoding: utf-8 -*-


def get_index_in_alphabet(char, alphabet):
    for j in range(len(alphabet)):
        try:
            alphabet[j].index(char)
            break
        except ValueError:
            pass
    return j


BINARY = u"01"
DECIMAL = u"0123456789"
DOZENAL = u"0123456789ab"
HEX = u"0123456789abcdef"
OCTAL = u"01234567"

ARABIC = u"غظضذخثتشرقصفعسنملكيطحزوهدجبأ"
DANISH = u"abcdefghijklmnopqrstuvwxyzæøå"
DUTCH = u"abcdefghijklmnopqrstuvwxyz"
ENGLISH = u"abcdefghijklmnopqrstuvwxyz"
GERMAN = u"abcdefghijklmnopqrstuvwxyzäöüß"
GREEK = u"αβγδεζηθικλμνξοπρστυφχψω"
HEBREW = u"אבגדהוזחטיךכלםמןנסעףפץצקרשת"
ICELANDIC = u"aábdðeéfghiíjklmnoóprstuúvxyýþæö"
ITALIAN = u"abcdefghilmnopqrstuvz"
NORWEGIAN = DANISH
POLISH = u"aąbcćdeęfghijklłmnńoóprsśtuwyzźż"
RUSSIAN = u"абвгдеёжзийклмнопрстуфхцчшщъыьэюя"
SPANISH = u"abcdefghijklmnñopqrstuvwxyz"
TURKISH = u"abcçdefgğhıijklmnoöprsştuüvyz"

ENGLISH_SQUARE_IJ = (
    u"a", u"b", u"c", u"d", u"e",
    u"f", u"g", u"h", u"ij", u"k",
    u"l", u"m", u"n", u"o", u"p",
    u"q", u"r", u"s", u"t", u"u",
    u"v", u"w", u"x", u"y", u"z",
)

ENGLISH_SQUARE_OQ = (
    u"a", u"b", u"c", u"d", u"e",
    u"f", u"g", u"h", u"i", u"j",
    u"k", u"l", u"m", u"n", u"oq",
    u"p", u"r", u"s", u"t", u"u",
    u"v", u"w", u"x", u"y", u"z",
)

ENGLISH_SQUARE_NO_Z = (
    u"a", u"b", u"c", u"d", u"e",
    u"f", u"g", u"h", u"i", u"j",
    u"k", u"l", u"m", u"n", u"o",
    u"p", u"q", u"r", u"s", u"t",
    u"u", u"v", u"w", u"x", u"y",
)

GERMAN_SQUARE = (
    u"aä", u"b", u"c", u"d", u"e",
    u"f", u"g", u"h", u"ij", u"k",
    u"l", u"m", u"n", u"oö", u"p",
    u"q", u"r", u"sß", u"t", u"uü",
    u"v", u"w", u"x", u"y", u"z"
)

SPANISH_SQUARE = (
    u"a", u"b", u"c", u"d", u"e",
    u"f", u"g", u"h", u"ij", u"k",
    u"l", u"m", u"nñ", u"o", u"p",
    u"q", u"r", u"s", u"t", u"u",
    u"v", u"w", u"x", u"y", u"z"
)

RUSSIAN_SQUARE = (
    u"а", u"б", u"в", u"г", u"д", u"её",
    u"ж", u"з", u"ий", u"к", u"л", u"м",
    u"н", u"о", u"п", u"р", u"с", u"т",
    u"у", u"ф", u"х", u"ц", u"ч", u"ш",
    u"щ", u"ы", u"ьъ", u"э", u"ю", u"я",
    u"0", u"1", u"2", u"3", u"4", u"5"
)

JAPANESE_HIRAGANA = (
    u"あいうえお"
    u"かきくけこ"
    u"がぎぐげご"
    u"さしすせそ"
    u"ざじずぜぞ"
    u"たちつてと"
    u"だぢづでど"
    u"なにぬねの"
    u"はひふへほ"
    u"ばびぶべぼ"
    u"ぱぴぷぺぽ"
    u"まみむめも"
    u"やゆよ"
    u"らりるれろ"
    u"わを"
    u"ん"
    u"ゃゅょぁぇ"
    u"じづ"
)



#!/usr/bin/python
# -*- encoding: utf-8 -*-


class Zigzag:
    """
    The Zigzag Cipher
    """

    def __enc(self, key, text):
        crypted = ""
        step = (key - 1) << 1
        textlen = len(text)

        # first row
        left = 0
        while (left < textlen):
            crypted += text[left]
            left += step

        # next rows
        for row in range(1, key):
            left = row
            while (left < textlen):
                crypted += text[left]
                right = left + step - (row << 1)
                if right < textlen and right != left:
                    crypted += text[right]
                left += step

        return crypted

    def __dec(self, key, text):
        step = (key - 1) << 1
        textlen = len(text)
        decrypted = ["."] * textlen

        # first row
        left = 0
        i = 0
        while (left < textlen):
            decrypted[left] = text[i]
            left += step
            i += 1

        # next rows
        for row in range(1, key):
            left = row
            while (left < textlen):
                decrypted[left] = text[i]
                i += 1
                right = left + step - (row << 1)
                if right < textlen and right != left:
                    decrypted[right] = text[i]
                    i += 1
                left += step

        return "".join(decrypted)

    def encrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__enc(key, text)

    def decrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__dec(key, text)



#!/usr/bin/python

from .adfgx import ADFGX
from .adfgvx import ADFGVX
from .affine import Affine
from .atbash import Atbash
from .autokey import Autokey
from .bazeries import Bazeries
from .beaufort import Beaufort
from .bifid import Bifid
from .caesar import Caesar
from .caesar_progressive import CaesarProgressive
from .columnar_transposition import ColumnarTransposition
from .four_square import FourSquare
from .gronsfeld import Gronsfeld
from .keyword import Keyword
from .nihilist import Nihilist
from .simplesubstitution import SimpleSubstitution
from .porta import Porta
from .playfair import Playfair
from .polybius import Polybius
from .rot13 import Rot13
from .rot18 import Rot18
from .rot47 import Rot47
from .rot5 import Rot5
from .three_square import ThreeSquare
from .trifid import Trifid
from .two_square import TwoSquare
from .vic import Vic
from .vigenere import Vigenere
from .zigzag import Zigzag

__all__ = [
    "ADFGX", "ADFGVX", "Affine", "Atbash", "Autokey",
    "Bazeries", "Beaufort", "Bifid", "Caesar", "CaesarProgressive",
    "ColumnarTransposition", "FourSquare", "Gronsfeld", "Keyword", "Nihilist",
    "SimpleSubstitution", "Playfair", "Porta", "Polybius", "Rot13",
    "Rot18", "Rot47", "Rot5", "ThreeSquare", "Trifid",
    "TwoSquare", "Vigenere", "Vic", "Zigzag",
]



#!/usr/bin/python
# -*- encoding: utf-8 -*-
from __future__ import division
import math
from .polybius import Polybius


class ADFGVX:
    """
    The ADFGVX Cipher
    """
    __header = u"adfgvx"
    __polybius = Polybius()
    __alphabet = [
        u"a", u"b", u"c", u"d", u"e", u"f",
        u"g", u"h", u"ij", u"k", u"l", u"m",
        u"n", u"o", u"p", u"q", u"r", u"s",
        u"t", u"u", u"v", u"w", u"x", u"y",
        u"z", u"1", u"2", u"3", u"4", u"5",
        u"6", u"7", u"8", u"9", u"0", u".",
    ]

    def __dec(self, alphabet, key, text, isEncrypt):
        keysize = len(key)
        size = len(text)
        rows = int(math.ceil(size/keysize))
        reminder = size % keysize
        keyword = list(key)
        indices = sorted(range(len(keyword)), key=lambda k: keyword[k])

        myarr = list(indices)
        lefti = 0
        righti = 0
        for key, value in enumerate(indices):
            righti = lefti
            righti += rows
            if reminder > 0 and value > reminder-1:
                righti -= 1
            myarr[value] = text[lefti:righti]
            lefti = righti

        column = 0
        row = 0
        res = ''
        for i in range(size):
            res += (myarr[column][row])
            column += 1
            if column == keysize:
                column = 0
                row += 1

        code = [str(self.__header.index(char)+1) for char in res]
        code = "".join(code)
        return self.__polybius.decrypt(code, alphabet=alphabet)

    def __enc(self, alphabet, key, text, isEncrypt):
        ans0 = self.__polybius.encrypt(text, alphabet=alphabet)
        ans = [self.__header[int(char)-1] for char in ans0]

        keyword = list(key)
        keysize = len(key)
        size = len(ans)
        indices = sorted(range(len(keyword)), key=lambda k: keyword[k])
        row = int(math.ceil(size/keysize))
        for i in range(row):
            p = ""
            for j in range(keysize):
                myi = i*keysize+j
                if myi < len(ans):
                    eee = ans[myi]
                    p += "".join(eee)

        ans2 = ""
        for s in indices:
            ind = s
            while ind < size:
                p = ans[ind]
                ans2 += p
                ind += keysize

        return ans2

    def encrypt(self, text, key, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or self.__alphabet
        return self.__enc(alphabet, key, text, 1)

    def decrypt(self, text, key, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or self.__alphabet
        return self.__dec(alphabet, key, text, -1)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .decorator import AbstractMachineDecorator


class SaveSpaces(AbstractMachineDecorator):

    def encrypt(self, text):
        return self.__encDec(text, lambda text: self._machine.encrypt(text))

    def decrypt(self, text):
        return self.__encDec(text, lambda text: self._machine.decrypt(text))

    def __encDec(self, text, func):
        spaces = []
        for i, char in enumerate(text):
            if char == ' ':
                spaces.append(i)
        text = text.replace(" ", "")
        res = func(text)
        for i in spaces:
            res = res[:i] + ' ' + res[i:]
        return res



#!/usr/bin/python
# -*- encoding: utf-8 -*-


class Autokey:
    """
    The Autokey Cipher
    """

    def __encDec(self, alphabet, key, text, isEncrypt):
        ans = ""
        for i in range(len(text)):
            m = text[i]
            if i < len(key):
                k = key[i]
            else:
                if isEncrypt == 1:
                    k = text[i - len(key)]
                else:
                    k = ans[i - len(key)]
            alphI = (
                    alphabet.index(m) + isEncrypt * alphabet.index(k)
                ) % len(alphabet)
            enc = alphabet[alphI]
            ans += enc
        return ans

    def encrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, 1)

    def decrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, -1)



#!/usr/bin/python
# -*- encoding: utf-8 -*-


class Trifid:
    """
    The Trifid Cipher
    """
    alphabet = [
        u"a", u"b", u"c",
        u"d", u"e", u"f",
        u"g", u"h", u"i",

        u"j", u"k", u"l",
        u"m", u"n", u"o",
        u"p", u"q", u"r",

        u"s", u"t", u"u",
        u"v", u"w", u"x",
        u"y", u"z", u".",
    ]

    def __code(self, text, alphabet):
        code = ""
        for char in text:
            for index in range(len(alphabet)):
                try:
                    alphabet[index].index(char)
                    break
                except ValueError:
                    pass
            square = int(index / 9)
            index = index % 9
            row = int(index / 3)
            col = index % 3
            code += str(square+1) + str(row+1) + str(col+1)
        return code

    def __decode(self, text, alphabet):
        code = ""
        for i in range(0, len(text), 3):
            square = int(text[i])-1
            row = int(text[i+1])-1
            col = int(text[i+2])-1
            index = square*9 + row*3 + col
            code += alphabet[index][0]
        return code

    def __enc(self, alphabet, text, key):
        code = self.__code(text, alphabet)

        code0 = ""
        for j in range(0, len(text)*3, 3*key):
            for i in range(3):
                code0 += code[j+i:j+3*key:3]

        code = self.__decode(code0, alphabet)
        return code

    def __dec(self, alphabet, text, key):
        code = self.__code(text, alphabet)

        code0 = ""
        rmd = (len(text) % key)
        for j in range(0, (len(text) - rmd) * 3, 3*key):
            for i in range(key):
                code0 += code[j+i:j+3*key:key]

        j = (len(text) - rmd) * 3
        for i in range(rmd):
            code0 += code[j+i:j+3*rmd:rmd]

        code = self.__decode(code0, alphabet)
        return code

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or self.alphabet
        key = int(key)
        if not key > 0:
            key = len(text)
        return self.__enc(alphabet, text, key)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or self.alphabet
        key = int(key)
        if not key > 0:
            key = len(text)
        return self.__dec(alphabet, text, key)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .polybius_square import PolybiusSquare


class TwoSquare:
    """
    The Two-Square Cipher
    """

    def __enc(self, alphabet, text, key):
        square1 = PolybiusSquare(alphabet, key[0])
        square2 = PolybiusSquare(alphabet, key[1])

        # text encryption
        if len(text) % 2:
            text += alphabet[-1][0]
        odd = text[1::2]
        even = text[::2]
        enc = u""

        for i in range(len(even)):
            coords = square1.get_coordinates(even[i])
            row1 = coords[0]
            column1 = coords[1]

            coords = square2.get_coordinates(odd[i])
            row2 = coords[0]
            column2 = coords[1]

            if column1 == column2:
                enc += square1.get_char(row2, column1)
                enc += square2.get_char(row1, column1)
            else:
                enc += square1.get_char(row1, column2)
                enc += square2.get_char(row2, column1)
        return enc

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__enc(alphabet, text, key)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__enc(alphabet, text, key)



#!/usr/bin/python
# -*- encoding: utf-8 -*-


class Vic:
    """
    The Vic Cipher
    """

    def __find_index_in_alphabet(self, char, alphabet):
        for j in range(len(alphabet)):
            try:
                alphabet[j].index(char)
                break
            except ValueError:
                pass
        return j

    def __encDec(self, alphabet, text, key, do_encrypt):
        columns = []
        width = 10
        # define columns with null string
        for i, value in enumerate(alphabet):
            if value == "":
                columns.append(i)

        # encode chars to numbers
        code = ""
        for char in text:
            j = self.__find_index_in_alphabet(char, alphabet)
            row = int(j / width)
            if row > 0:
                column = j % width
                code += str(columns[row-1]) + str(column)
            else:
                code += str(j)

        enc = ""
        if do_encrypt:
            # addition by key
            for i in range(0, len(code)):
                enc += str((int(code[i]) + int(key[i % len(key)])) % 10)
        else:
            # subraction by key
            for i in range(0, len(code)):
                enc += str((int(code[i]) - int(key[i % len(key)])) % 10)

        # encode numbers to chars
        enc2 = ""
        row = 0
        for i in range(0, len(enc)):
            if row == 0 and (int(enc[i]) in columns):
                row = columns.index(int(enc[i])) + 1
            else:
                enc2 += alphabet[row * width + int(enc[i])][0]
                row = 0
        return enc2

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__encDec(alphabet, text, key, True)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u'ij', u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__encDec(alphabet, text, key, False)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .polybius_square import PolybiusSquare


class Bazeries:
    """
    The Bazeries Cipher
    """

    def __encDec(self, alphabet, text, key, isEncrypt=True):
        square1 = PolybiusSquare(alphabet)

        # key is a number, make it a string
        square2 = PolybiusSquare(alphabet, key[1])

        # prepare text: group and reverse
        temp = key[0]
        groups = []
        while temp > 0:
            rmd = temp % 10
            temp = int(temp / 10)
            groups.append(rmd)
        groups = groups[::-1]

        i = 0
        j = 0
        revtext = ""
        while i < len(text):
            num = groups[j]
            str1 = text[int(i):int(i+num)]
            revtext += str1[::-1]
            i += num
            j += 1
            if j == len(groups):
                j = 0

        # now we have reversed text and we encrypt
        ret = ""
        if isEncrypt:
            for char in revtext:
                coords = square1.get_coordinates(char)
                ret += square2.get_char(coords[1], coords[0])
        else:
            for char in revtext:
                coords = square2.get_coordinates(char)
                ret += square1.get_char(coords[1], coords[0])
        return ret

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__encDec(alphabet, text, key, True)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__encDec(alphabet, text, key, False)



#!/usr/bin/python


class Affine:
    """
    The Affine Cipher
    """

    def __encDec(self, alphabet, key, text, isEncrypt):
        a = key[0]
        b = key[1]
        ans = ""
        aInverse = self.__getInverse(a, alphabet)
        for char in text:
            if isEncrypt == 1:
                alphI = (alphabet.index(char) * a + b) % len(alphabet)
            else:
                alphI = (aInverse * (alphabet.index(char) - b)) % len(alphabet)
            enc = alphabet[alphI]
            ans += enc
        return ans

    def __getInverse(self, a, alphabet):
        for i in range(1, len(alphabet)):
            if ((int(a)*int(i)) % int(len(alphabet))) == 1:
                return i
        return 0

    def encrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, 1)

    def decrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, -1)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .polybius import Polybius


class Bifid:
    """
    The Bifid Cipher
    """

    __polybius = Polybius()

    def __enc(self, alphabet, text, key):
        code = self.__polybius.encrypt(text, alphabet=alphabet)
        even = code[::2]
        odd = code[1::2]
        ret = ""
        for i in range(0, len(even), key):
            ret += even[i:i+key] + odd[i:i+key]
        return self.__polybius.decrypt(ret, alphabet=alphabet)

    def __dec(self, alphabet, text, key):
        code = self.__polybius.encrypt(text, alphabet=alphabet)
        even = ""
        odd = ""
        rem = len(code) % (key << 1)
        for i in range(0, len(code)-rem, key << 1):
            ikey = i+key
            even += code[i:ikey]
            odd += code[ikey:ikey+key]

        even += code[-rem:-(rem >> 1)]
        odd += code[-(rem >> 1):]

        code = ""
        for i in range(len(even)):
            code += even[i] + odd[i]
        return self.__polybius.decrypt(code, alphabet=alphabet)

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        key = int(key)
        if not key > 0:
            key = len(text)
        return self.__enc(alphabet, text, key)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u'ij', u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        key = int(key)
        if not key > 0:
            key = len(text)
        return self.__dec(alphabet, text, key)



#!/usr/bin/python

from .uppercase import UpperCase
from .lowercase import LowerCase
from .nospaces import NoSpaces
from .savespaces import SaveSpaces
from .savecase import SaveCase

__all__ = [
    "UpperCase", "LowerCase", "NoSpaces", "SaveSpaces", "SaveCase",
]



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .decorator import AbstractMachineDecorator


class UpperCase(AbstractMachineDecorator):

    def encrypt(self, text):
        text = text.lower()
        res = self._machine.encrypt(text)
        return res.upper()

    def decrypt(self, text):
        text = text.lower()
        res = self._machine.decrypt(text)
        return res.upper()



#!/usr/bin/python


class Atbash:
    """
    The Atbash Cipher
    """

    def __encDec(self, alphabet, text):
        ans = ""
        for char in text:
            alphIndex = len(alphabet) - (alphabet.index(char)) - 1
            enc = alphabet[alphIndex]
            ans += enc
        return ans

    def encrypt(self, text, key=None, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, text)

    def decrypt(self, text, key=None, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, text)



#!/usr/bin/python
# -*- encoding: utf-8 -*-
from secretpy import alphabets


class ColumnarTransposition:
    """
    The Columnar Transposition Cipher
    """

    def __dec(self, alphabet, key, text):
        chars = [alphabets.get_index_in_alphabet(char, alphabet)
                 for char in key]
        keyorder = sorted(enumerate(chars), key=lambda x: x[1])
        ret = u""
        rows = int(len(text) / len(key))

        cols = [0] * len(key)
        for i, item in enumerate(range(0, len(text), rows)):
            cols[keyorder[i][0]] = text[item: item + rows]

        for j in range(rows):
            for i in range(len(cols)):
                ret += cols[i][j]
        return ret

    def __enc(self, alphabet, key, text):
        # add endings to the text to fill the square
        rmd = len(text) % len(key)
        if rmd != 0:
            text += alphabet[-1] * (len(key) - rmd)

        chars = [alphabets.get_index_in_alphabet(char, alphabet)
                 for char in key]
        keyorder1 = sorted(enumerate(chars), key=lambda x: x[1])

        ret = u""
        for i in range(len(key)):
            ret += text[keyorder1[i][0]::len(key)]

        return ret

    def encrypt(self, text, key, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or alphabets.ENGLISH
        return self.__enc(alphabet, key, text)

    def decrypt(self, text, key, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used, if there is no a value,
                         English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or alphabets.ENGLISH
        return self.__dec(alphabet, key, text)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .polybius_square import PolybiusSquare


class Playfair:
    """
    The Playfair Cipher
    """

    def __dec_two_letters(self, a, b, square):
        cols = square.get_columns()
        rows = square.get_rows()

        coords = square.get_coordinates(a)
        arow = coords[0]
        acolumn = coords[1]

        coords = square.get_coordinates(b)
        brow = coords[0]
        bcolumn = coords[1]

        if arow == brow:
            a = square.get_char(arow, (acolumn - 1) % cols)
            b = square.get_char(brow, (bcolumn - 1) % cols)
        elif acolumn == bcolumn:
            arow -= 1
            if arow >= rows:
                arow = 0
            a = square.get_char(arow, acolumn)

            brow -= 1
            if brow >= rows:
                brow = 0
            b = square.get_char(brow, bcolumn)
        else:
            a = square.get_char(arow, bcolumn)
            b = square.get_char(brow, acolumn)
        return a + b

    def __enc_two_letters(self, a, b, square):
        cols = square.get_columns()
        rows = square.get_rows()

        coords = square.get_coordinates(a)
        arow = coords[0]
        acolumn = coords[1]

        coords = square.get_coordinates(b)
        brow = coords[0]
        bcolumn = coords[1]

        if arow == brow:
            a = square.get_char(arow, (acolumn + 1) % cols)
            b = square.get_char(brow, (bcolumn + 1) % cols)
        elif acolumn == bcolumn:
            arow += 1
            if arow >= rows:
                arow = 0
            a = square.get_char(arow, acolumn)
            brow += 1
            if brow >= rows:
                brow = 0
            b = square.get_char(brow, bcolumn)
        else:
            a = square.get_char(arow, bcolumn)
            b = square.get_char(brow, acolumn)
        return a + b

    def __enc(self, alphabet, text, key):
        enc = u""
        insert_char = 'x'
        i = 1
        square = PolybiusSquare(alphabet, key)

        while i < len(text):
            if text[i-1] == text[i]:
                text = text[:i] + insert_char + text[i:]
            enc += self.__enc_two_letters(text[i-1], text[i], square)
            i += 2

        if len(text) & 1:
            enc += self.__enc_two_letters(text[-1], insert_char, square)

        return enc

    def __dec(self, alphabet, text, key):
        dec = ""
        insert_char = 'x'

        square = PolybiusSquare(alphabet, key)

        for i in range(1, len(text), 2):
            pair = self.__dec_two_letters(text[i-1], text[i], square)
            if len(dec) > 1 and dec[-1] == insert_char and pair[0] == dec[-2]:
                dec = dec[:-1] + pair[0]
                dec += pair[1]
            else:
                dec += self.__dec_two_letters(text[i-1], text[i], square)
        if dec[-1] == insert_char:
            dec = dec[:-1]
        return dec

    def encrypt(self, text, key="", alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__enc(alphabet, text, key)

    def decrypt(self, text, key="", alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__dec(alphabet, text, key)



#!/usr/bin/python
# -*- encoding: utf-8 -*-
from collections import OrderedDict


class Keyword:
    """
    The Keyword Cipher
    """

    def __removeDup(self, input_str):
        newstring = input_str[0]
        for i in range(len(input_str)):
            if newstring[(len(newstring) - 1)] != input_str[i]:
                newstring += input_str[i]
            else:
                pass
        return newstring

    def __encDec(self, alphabet, key, text, isEncrypt):
        # remove repeats of letters in the key
        newkey = "".join(OrderedDict.fromkeys(key))
        # create the substitution string
        longkey = "".join(OrderedDict.fromkeys(newkey+"".join(alphabet)))
        # do encryption
        ans = ""
        for i in range(len(text)):
            m = text[i]
            if isEncrypt == 1:
                index = alphabet.index(m)
                enc = longkey[index]
            else:
                index = longkey.index(m)
                enc = alphabet[index]
            ans += enc
        return ans

    def encrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, 1)

    def decrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, -1)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

import secretpy.alphabets as al


class Caesar:
    """
    The Caesar Cipher
    """
    __alphabet = al.ENGLISH

    def __encDec(self, alphabet, key, text, isEncrypt):
        alphabet = alphabet or self.__alphabet
        ans = ""
        for char in text:
            try:
                alphIndex = alphabet.index(char)
            except ValueError as e:
                wrchar = char.encode('utf-8')
                e.args = (
                    "Can't find char '" + wrchar + "' of text in alphabet!",)
                raise
            alphIndex = (alphIndex + isEncrypt * key) % len(alphabet)
            ans += alphabet[alphIndex]
        return ans

    def encrypt(self, text, key, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: encrypted text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, 1)

    def decrypt(self, text, key, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: decrypted text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, -1)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .polybius_square import PolybiusSquare


class FourSquare:
    """
    The Four-Square Cipher
    """

    def __enc(self, alphabet, text, key, isEncrypt):
        square01 = PolybiusSquare(alphabet, key[0])
        square10 = PolybiusSquare(alphabet, key[1])
        square = PolybiusSquare(alphabet, "")

        # text encryption
        if len(text) % 2:
            text += alphabet[-1][0]
        odd = text[1::2]
        even = text[::2]
        enc = u""
        if isEncrypt:
            for i in range(len(even)):
                coords = square.get_coordinates(even[i])
                row00 = coords[0]
                column00 = coords[1]

                coords = square.get_coordinates(odd[i])
                row11 = coords[0]
                column11 = coords[1]

                enc += square01.get_char(row00, column11)
                enc += square10.get_char(row11, column00)
        else:
            for i in range(len(even)):
                coords = square01.get_coordinates(even[i])
                row00 = coords[0]
                column00 = coords[1]

                coords = square10.get_coordinates(odd[i])
                row11 = coords[0]
                column11 = coords[1]

                enc += square.get_char(row00, column11)
                enc += square.get_char(row11, column00)
        return enc

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__enc(alphabet, text, key, True)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__enc(alphabet, text, key, False)



#!/usr/bin/python

from .rot13 import Rot13
import secretpy.alphabets as al


class Rot5:
    """
    The Rot5 Cipher
    """
    __rot13 = Rot13()
    __alphabet = al.DECIMAL

    def __encDec(self, text):
        return self.__rot13.encrypt(text, alphabet=self.__alphabet)

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(text)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(text)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

import math
from collections import OrderedDict


class PolybiusSquare:
    """
    PolybiusSquare. It's used by many classical ciphers
    """
    __alphabet = None
    __side = 0

    def __init__(self, alphabet, key=""):
        keyi = []
        if key:
            for char in key:
                index = self.__find_index_in_alphabet(char, alphabet)
                keyi.append(index)
            # remove duplicates
            keyi = OrderedDict.fromkeys(keyi)

        alph_out = []
        for i in keyi:
            alph_out.append(alphabet[i])

        for i in range(len(alphabet)):
            if i not in keyi:
                alph_out.append(alphabet[i])

        self.__alphabet = alph_out
        self.__side = int(math.ceil(math.sqrt(len(alphabet))))

    def __find_index_in_alphabet(self, char, alphabet):
        for j in range(len(alphabet)):
            try:
                alphabet[j].index(char)
                break
            except ValueError:
                pass
        return j

    def get_coordinates(self, char):
        for j in range(len(self.__alphabet)):
            try:
                self.__alphabet[j].index(char)
                break
            except ValueError:
                pass

        row = int(j / self.__side)
        column = j % self.__side
        return (row, column)

    def get_char(self, row, column):
        return self.__alphabet[row * self.__side + column][0]

    def get_columns(self):
        return self.__side

    def get_rows(self):
        return int(len(self.__alphabet) / self.__side)



#!/usr/bin/python
# -*- encoding: utf-8 -*-


class SimpleSubstitution:
    """
    The Simple Substitution Cipher
    """

    def __encDec(self, alphabet, key, text, isEncrypt):
        if len(alphabet) != len(key):
            return

        ans = ""
        for i in range(len(text)):
            m = text[i]
            k = ""
            if isEncrypt == 1:
                k = key[alphabet.index(m)]
            else:
                k = alphabet[key.index(m)]
            ans += k
        return ans

    def encrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, 1)

    def decrypt(self, text, key, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text, -1)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .decorator import AbstractMachineDecorator


class SaveCase(AbstractMachineDecorator):

    def encrypt(self, text):
        return self.__encDec(text, lambda text: self._machine.encrypt(text))

    def decrypt(self, text):
        return self.__encDec(text, lambda text: self._machine.decrypt(text))

    def __encDec(self, text, func):
        uppercases = []
        for i, char in enumerate(text):
            if char == char.upper():
                uppercases.append(i)
        text = text.lower()
        res = func(text)
        for i in uppercases:
            res = res[:i] + res[i].upper() + res[i+1:]
        return res



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .polybius import Polybius


class Nihilist:
    """
    The Nihilist Cipher
    """

    __polybius = Polybius()

    def __enc(self, alphabet, text, key):
        code = self.__polybius.encrypt(text, alphabet=alphabet)
        enc = ""
        for i in range(0, len(code), 2):
            char = self.__polybius.encrypt(key[(i >> 1) % len(key)],
                                           alphabet=alphabet)
            enc += str(int(code[i:i+2]) + int(char)) + " "
        return enc.rstrip()

    def __dec(self, alphabet, text, key):
        code = text.split(' ')
        code = list(map(int, code))
        dec = ""
        for i in range(0, len(code)):
            char = self.__polybius.encrypt(key[i % len(key)],
                                           alphabet=alphabet)
            pair = str(code[i] - int(char))
            dec += self.__polybius.decrypt(pair, alphabet=alphabet)
        return dec

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__enc(alphabet, text, key)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u'ij', u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__dec(alphabet, text, key)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from secretpy import alphabets


class Porta:
    """
    The Porta Cipher
    """
    def __encDec(self, alphabet, key, text):
        ans = ""
        for i, char in enumerate(text):
            keychari = alphabet.index(key[i % len(key)]) >> 1
            textindex = alphabet.index(char)
            half = len(alphabet) >> 1
            half_alphabet = None
            if textindex < half:
                half_alphabet = alphabet[half:]
                alphIndex = (textindex + keychari) % half
            else:
                half_alphabet = alphabet[0:half]
                alphIndex = (textindex - keychari) % half
            ans += half_alphabet[alphIndex]
        return ans

    def encrypt(self, text, key, alphabet=alphabets.ENGLISH):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, key, text)

    def decrypt(self, text, key, alphabet=alphabets.ENGLISH):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """

        return self.__encDec(alphabet, key, text)



#!/usr/bin/python

from .caesar import Caesar
import secretpy.alphabets as al


class Rot13:
    """
    The Rot13 Cipher
    """
    __caesar = Caesar()
    __alphabet = al.ENGLISH

    def __encDec(self, alphabet, text):
        alphabet = alphabet or self.__alphabet
        key = len(alphabet) >> 1
        # if number letters in the alphabet is odd
        if len(alphabet) & 1:
            alphabet += alphabet[key]
            key += 1
        return self.__caesar.encrypt(text, key, alphabet)

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, text)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        return self.__encDec(alphabet, text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="secretpy",
    version="0.8.0",
    author="Max Vetrov",
    author_email="maxvetrov555@yandex.ru",
    description="Classical ciphers",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/tigertv/secretpy",
    packages=setuptools.find_packages(exclude=['tests', 'tests.*']),
    test_suite='tests',
    project_urls={
        "Source code": "https://github.com/tigertv/secretpy",
        "Documentation": "https://secretpy.readthedocs.io",
        "Bug tracker": "https://github.com/tigertv/secretpy/issues",
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Other Audience',
        'Topic :: Security :: Cryptography',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Education',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    keywords="classic ciphers cipher secret",
)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .decorator import AbstractMachineDecorator


class NoSpaces(AbstractMachineDecorator):

    def encrypt(self, text):
        text = text.replace(" ", "")
        res = self._machine.encrypt(text)
        return res

    def decrypt(self, text):
        text = text.replace(" ", "")
        res = self._machine.decrypt(text)
        return res



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from ..abstractmachine import AbstractCryptMachine
from secretpy import alphabets


class AbstractMachineDecorator(AbstractCryptMachine):
    def __init__(self, machine):
        self._machine = machine

    def set_key(self, key):
        self._machine.set_key(key)

    def set_alphabet(self, alphabet=alphabets.ENGLISH):
        self._machine.set_alphabet(alphabet)

    def set_cipher(self, cipher):
        self._machine.set_cipher(cipher)

    def encrypt(self, text):
        pass

    def decrypt(self, text):
        pass



#!/usr/bin/python


class AbstractCryptMachine:
    def set_key(self, key):
        pass

    def set_alphabet(self, alphabet=u"abcdefghijklmnopqrstuvwxyz"):
        pass

    def set_cipher(self, cipher):
        pass

    def encrypt(self, text):
        pass

    def decrypt(self, text):
        pass



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .polybius_square import PolybiusSquare


class Polybius:
    """
    The Polybius Cipher
    """

    def __encDec(self, alphabet, text, key, isEncrypt=True):
        square = PolybiusSquare(alphabet, key)
        res = ""
        header = range(1, square.get_columns() + 1)
        header = "".join(map(str, header))
        if isEncrypt:
            for char in text:
                coords = square.get_coordinates(char)
                row = coords[0]
                column = coords[1]
                res += header[row] + header[column]
        else:
            for i in range(0, len(text), 2):
                row = header.index(text[i])
                column = header.index(text[i+1])
                res += square.get_char(row, column)
        return res

    def encrypt(self, text, key="", alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or (
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        )
        return self.__encDec(alphabet, text, key, True)

    def decrypt(self, text, key="", alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or (
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        )
        return self.__encDec(alphabet, text, key, False)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .polybius_square import PolybiusSquare
import random


class ThreeSquare:
    """
    The Three Square Cipher
    """

    def __encDec(self, alphabet, text, key, isEncrypt):
        square1 = PolybiusSquare(alphabet, key[0])
        square2 = PolybiusSquare(alphabet, key[1])
        square3 = PolybiusSquare(alphabet, key[2])

        enc = u""
        if isEncrypt:
            if len(text) % 2:
                text += alphabet[-1][0]

            odd = text[1::2]
            even = text[::2]

            for i in range(len(even)):
                row1, column1 = square1.get_coordinates(even[i])
                row2, column2 = square2.get_coordinates(odd[i])

                rows = square1.get_rows()
                index = random.randrange(rows)
                left = square1.get_char(index, column1)

                middle = square3.get_char(row1, column2)

                cols = square2.get_columns()
                index = random.randrange(cols)
                right = square2.get_char(row2, index)

                enc += left + middle + right
        else:
            trigrams = []
            i = 0
            while i < len(text):
                trigrams.append(text[i:i+3])
                i += 3

            for trigram in trigrams:
                col1 = square1.get_coordinates(trigram[0])[1]
                row3, col3 = square3.get_coordinates(trigram[1])
                row2 = square2.get_coordinates(trigram[2])[0]
                enc += square1.get_char(row3, col1)
                enc += square2.get_char(row2, col3)
        return enc

    def encrypt(self, text, key=None, alphabet=None):
        """
        Encryption method

        :param text: Text to encrypt
        :param key: Encryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__encDec(alphabet, text, key, True)

    def decrypt(self, text, key=None, alphabet=None):
        """
        Decryption method

        :param text: Text to decrypt
        :param key: Decryption key
        :param alphabet: Alphabet which will be used,
                         if there is no a value, English is used
        :type text: string
        :type key: integer
        :type alphabet: string
        :return: text
        :rtype: string
        """
        alphabet = alphabet or [
            u"a", u"b", u"c", u"d", u"e",
            u"f", u"g", u"h", u"ij", u"k",
            u"l", u"m", u"n", u"o", u"p",
            u"q", u"r", u"s", u"t", u"u",
            u"v", u"w", u"x", u"y", u"z"
        ]
        return self.__encDec(alphabet, text, key, False)



#!/usr/bin/python

from .abstractmachine import AbstractCryptMachine


class CompositeMachine(AbstractCryptMachine):
    __machines = []

    def __init__(self, *machines):
        for machine in machines:
            self.__machines.append(machine)

    def __encDec(self, text, func):
        ret = text
        for machine in self.__machines:
            ret = func(machine, ret)
        return ret

    def encrypt(self, text):
        return self.__encDec(text, lambda machine, text: machine.encrypt(text))

    def decrypt(self, text):
        return self.__encDec(text, lambda machine, text: machine.decrypt(text))

    def add_machines(self, *machines):
        for machine in machines:
            self.__machines.append(machine)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

from .decorator import AbstractMachineDecorator


class LowerCase(AbstractMachineDecorator):

    def encrypt(self, text):
        return self._machine.encrypt(text.lower())

    def decrypt(self, text):
        return self._machine.decrypt(text.lower())
